function [decimalExp, sampEntriesComparitiveArea] = SampEntriesAMOEBA(M, V_SampEntriesStatusQuo, V_SampEntriesScenario)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% This code produces an AMOEBA plot using sample entries data and         %
% principal component analysis output calculated here by princomp.m found %
% in MATLAB's Statistics package.  The style of plot is similar to that   %
% found in the work of Collie et al. (2003).  The code takes in as input  %
% a matrix of values involving two data groups.  Following suit from the  % 
% Collie et al. (2003) paper, we have the two groups 'fleets' and         %
% 'speices'.  In this case, the group 'species' is associated with sample %
% entries simply because that group has more variables than the fleets    %
% group (number of fleets = 8, number of species = 10). Had the           %
% magnitudes been switched, the species would be associated with the      %
% variables and the fleets with sample entries.                           %
%                                                                         %
% Also taken in as input, this code requires two vectors.  Working from   %
% our example, these vectors are measurements associated with our         %
% species, and thus each have ten (10) entries, and in order to be used   % 
% in this code's calculations, they must be column vectors.  The first    %
% vector needs to contain what Collie et al. (2003) refers to as a        %
% 'status quo' measurement.  We will also need a 'scenario' measurement   %
% vector to have something to compare to the 'status quo'.                %
%                                                                         %
% The vector division of these two vectors (i.e., scenario vector divided %
% by status quo vector) produces the length of the vectors we plot in the %
% figure.  These ten (10) vectors, one for each species, show the         % 
% relative affect the scenario had on the system (which we assume would   %
% be at status quo in lieu of the perturbation implied by said scenario). %
% If there is no effect due to the scenario, the vector will be of length %
% one (1) and will lie on the reference circle, which is of radius        % 
% one (1).                                                                %
%                                                                         %
% The direction in which the plotted vector is pointing is caclulated     %
% from output produced by princomp.m and will point in the same direction %
% as the fleets plotted vectors that catch them, which arise from running %
% VariablesAMOEBA.m.                                                      %
%                                                                         %
% For convenience, included in this package of .m files is the parent     %
% code BothAMOEBAS.m which run VariablesAMOEBA.m and SampEntriesAMOEBA.m  %
% simulaniously and produces AMOEBA plots for variables and sample        %
% entries (fleets and species, respectively) over a single scenario.  To  %
% run BothAMOEBAS.m, of course one will require status quo and scenario   %
% mearsurment vectors for both variables (fleets) and sample entries      %
% (species).                                                              %
%                                                                         %
% For more informantion, including an example with an example data matrix %
% and example data vectors, see the AMOEBAReadMe PDF document included    %
% with this code.                                                         %
%                                                                         %
% SampEntriesAMOEBA.m                                                     %
% Written by: Iris A. Gray, M.S.                                          %
%             Marine Ecosystem Modeller                                   %
%             NWFSC (NOAA)                                                %
%             Seattle, WA                                                 %
%             Iris.Gray@noaa.gov                                          %
%             9/9/2010                                                    %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% First things first, lets assign names to your SampEntries that you want 
% to see in the figure that this code produces.  The three letters you see 
% after the 's.' will be the name printed on its corresponding vector in 
% the figure.  To clarify further, the way this code is currently 
% structured, we have ten species named COD, HADDOCK, WHITING, SAITHE, 
% PLAICE, SOLE, HERRING, MACKEREL, SANDEEL, AND NORWAYPOUT.  Change these 
% names to whatever you desire.  Also, feel free to add or remove names as 
% necessary, but be sure to maintain the structural convention. 

s.COD = 1;
s.HADDOCK = 2;
s.WHITING = 3;
s.SAITHE = 4;
s.PLAICE = 5;
s.SOLE = 6;
s.HERRING = 7;
s.MACKEREL = 8;
s.SANDEEL = 9;
s.NORWAYPOUT = 10;


% Here we make sure that the user entered their data matrix correctly,
% i.e., that the number of rows is greater than or equal to the number of
% columns.  If the user enters a matrix in which their are fewer rows than
% columns, an error message is displayed which includes a solution to the
% error.

[r,c] = size(M);

if r<c
    error('User has entered a matrix of size n-by-p such that n < p.  Our PCA calculator (princomp.m) requires that the input matrix must have at least as many rows as columns.  To fix the error, transpose the input matrix.  Be sure to match the names you assigned to your variables and sample entries accordingly.');
end



% From MATLAB's princomp.m (rewritten to fit the current circumstances): 
% The command princomp performs principal components analysis on the n-by-p 
% data matrix M, and calculates the principal component coefficients, we 
% call variables. Rows of M correspond to observations, columns to 
% variables. variables is a p-by-p matrix, each column containing 
% coefficients for one principal component. The columns are in order of 
% decreasing component variance. Also provided here are the sample entries, 
% the principal component scores; that is, the representation of M in the 
% principal component space. Rows of SCORE correspond to observations, 
% columns to components.  Lastly, we have coveig, a vector containing the 
% eigenvalues of the covariance matrix of M.  Note: where variables would
% be, we have a 'junk' which denotes that this information is not used in this
% code.

[junk, SampEntries, coveig] = princomp(M);


% Here we caclulate the percentage of variance explained by our
% first two principal components.

decimalExp = cumsum(coveig)./sum(coveig);
decimalExp = decimalExp(2);


% Vector stuff.

p0 = [0,0];  % Initial point of every vector.
m = length(SampEntries);   % Calculates number of rows for SampEntries 
                           % matrix.


% In order to draw a dashed outline around the vector endpoints, we must
% first order them so that patch.m knows what order to draw the vertices.
% This bit first converts all the SampEntries into polar form and then 
% grabs the thetas and throws them into the first column of an n-by-2 
% matrix called polarVector2.

SampEntries = SampEntries(:,1:2);
for p = 1:m
    [theta2,junk] = cart2pol(SampEntries(p,1),SampEntries(p,2));
    polarVector2(p,1) = theta2;
end


% Here we are caclulating the lengths of the vectors in which we will plot.

polarVector2(:,2) = V_SampEntriesScenario./V_SampEntriesStatusQuo;


% This counts how many positive and negative thetas we have which aide in
% ordering them properly.

pos2 = 0;
neg2 = 0;

for u = 1:m
    if polarVector2(u,1)>0
        pos2 = pos2 + 1;
    else
        neg2 = neg2 + 1;
    end
end


% When we produce the plotted vectors, we want the ordering of them to 
% match the ordering of the names we gave them at the beginning of this 
% code.  Thus, newvariables is the matrix of x and y values which are the 
% plotted vector endpoints in the order corresponding to their vector 
% names.

for i  = 1:m
    [xCART,yCART] = pol2cart(polarVector2(i,1), polarVector2(i,2));
    newSampEntries(i,1) = xCART;
    newSampEntries(i,2) = yCART;
end



% Here, we seperate our positive and negative SampEntries again, for the
% purpose of ordering them properly.  This needs to be done so 'patch' can
% draw the polygon around the vector endpoints properly.

for i = 1:m
    if polarVector2(i,1)>0
        posSampEntries(i,1:2) = polarVector2(i,1:2);
    else
        posSampEntries(i,1:2) = [0,0];
    end
    if polarVector2(i,1)<0
        negSampEntries(i,1:2) = polarVector2(i,1:2);
    else 
        negSampEntries(i,1:2) = [0,0];
    end
end 


% Shrinking down our vectors to their nonzero elements.

pPS = zeros(pos2,2);
nPS = zeros(neg2,2);
pPS(:,1) = nonzeros(posSampEntries(:,1));
pPS(:,2) = nonzeros(posSampEntries(:,2));
nPS(:,1) = nonzeros(negSampEntries(:,1));
nPS(:,2) = nonzeros(negSampEntries(:,2));



% Sorting our polarVector2 entries according to theta value for both 
% positive and negative theta values seperately.  Each theta's
% corresponding rho (entries resulting from the vector division we saw in 
% line 128 of this code) is sorted according to the sorted placement of its
% theta.

  [junk, NewRowNumberP2] = sort(pPS(:,1));
  sortedPPS = pPS(NewRowNumberP2,:);
  [junk, NewRowNumberN2] = sort(nPS(:,1));
  sortedNPS = nPS(NewRowNumberN2,:);
  

% Now, we can throw all of our sortedPPS and sortedNPS together into 
% sortedPS such that they are ordered according to their theta values, so 
% that they sweep around the origin from 0 to 2*pi.
  
  sortedPS = zeros(m,2);
  sortedPS(1:pos2,:) = sortedPPS;
  sortedPS(pos2+1:m,:) = sortedNPS;
  
  
% In order to plot these sorted polar coordinates in a Cartesian plane, we
% must convert:
  
for i  = 1:m
    [xCart,yCart] = pol2cart(sortedPS(i,1), sortedPS(i,2));
    [xCART2,yCART2] = pol2cart(sortedPS(i,1), 1);  % We additionally need 
                                                   % to caclulate what the
                                                   % AMOEBA area would be
                                                   % under the status quo
                                                   % scenaro.
    sortedPSCart(i,1) = xCart;
    sortedPSCart(i,2) = yCart;
    comparePSCart(i,1) = xCART2;                   % Status quo comparison 
    comparePSCart(i,2) = yCART2;                   % ordered vector
                                                   % endpoints.
    
end
 

% Here we calculate the area of the polygon (or AMOEBA) and also what 
% percentage of its status quo AMOEBA's area it encompasses.  
  
  SampEntriesAmoebaArea = polyarea(sortedPSCart(:,1),sortedPSCart(:,2));
  statqSEAmoebaArea = polyarea(comparePSCart(:,1),comparePSCart(:,2));
  h = 0; k = 0; r = 1;    % Circle stuff.
  N = 256;                % Circle stuff.
  t = (0:N)*2*pi/N;       % Circle stuff.
  sampEntriesComparitiveArea = SampEntriesAmoebaArea/statqSEAmoebaArea;
  

% SampEntries figure!  Yay!!! 
  
f2 = figure(2);


% This bit loads the names of the variables we chose at the beginning of the
% code.

sn = fieldnames(s); 


% This command, 'patch' is what draws the polygon (or AMOEBA) whose 
% verticies are the endpoints of the plotted vectors.  As mentioned
% before, we use sortedPSCart as opposed to newSampEntries to draw the 
% patch since the vector endpoints are ordered in a way that they sweep 
% around the origin.  Using newSampEntries, although having the same values
% as sortedPSCart, would more often than not draw a crazy looking polygon
% since the vertices of the shape would probably not be drawn in the 
% correct order.

patch(sortedPSCart(:,1),sortedPSCart(:,2), 'w', 'EdgeColor', 'k', 'LineStyle', '--')

% Be sure to change title if needed!

title('Species AMOEBA', 'FontSize', 12)


% Here, we draw the reference circle and plot our SampEntries vectors, with 
% names.  Please note that the placement of the names might not be 
% aesthetically pleasing in the figure produced.  If you wish to move the 
% name of the vector then within the figure space, simply go to 'edit' and 
% then 'figure properties'which will allow you to move the vector names 
% wherever you please.

for j = 1:m
    hold on

    plot( r*cos(t)+h, r*sin(t)+k);   % Plots the reference circle
    
    % Plots vectors and corresponding names according to quadrant of the
    % Cartesian plane.
    
    vectarrow(p0, [newSampEntries(j,1), newSampEntries(j,2)]);
    %if newSampEntries(j,2)>0 & newSampEntries(j,1)>0
        text(newSampEntries(j,1), newSampEntries(j,2), sn(j))
    %elseif newSampEntries(j,2)<0 & newSampEntries(j,1)>0
     %   text(newSampEntries(j,1), newSampEntries(j,2)+0.05, sn(j))
    %elseif newSampEntries(j,2)<0 & newSampEntries(j,1)<0
    %    text(newSampEntries(j,1)-0.21, newSampEntries(j,2)-0.05, sn(j))
    %elseif newSampEntries(j,2)>0 & newSampEntries(j,1)<0
    %    text(newSampEntries(j,1)-0.21, newSampEntries(j,2)+0.03, sn(j))
    %end
    grid off
    axis off
end

% This command saves the figure produced as SampEntriesAMOEBA.pdf.  Please
% note that the figure may not pop up with a perfectly round reference
% circle, and therefore, the .pdf of the figure will then be an exact
% snapshot of whatever the figure looks like initially, which may be 
% undesired if the reference circle comes out looking like an elipse.  This
% can be easily remedied by stretching the sides of the figure viewing 
% window so that the reference circle looks circular, rather than 
% eliptical.  Then, save the figure, choosing the name that the figure was 
% already given (SampEntriesAMOEBA.pdf).  You will then be prompted to 
% override the previous version, in which you definitely want to do so.

print(f2,'-dpdf','SampEntriesAMOEBA.pdf')