function [decimalExp, variablesComparitiveArea] = VariablesAMOEBA(M, V_variablesStatusQuo, V_variablesScenario)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                         %
% This code produces an AMOEBA plot using the loadings produced via       %
% principal component analysis implemented here by princomp.m found in    %
% MATLAB's Statistics package to calculate plotted vectors' angles, and   %
% using the variables status quo and scenario measurements to calculate   %
% the length of the plotted vectors.  The style of plot is similar to     %
% that found in the work of Collie et al. (2003).  The parent code        %
% BothAMOEBAS.m takes in as input a matrix of values involving two groups %
% of data.  Following suit from the Collie et al. (2003) paper, we have   %
% the two groups 'fleets' and 'speices'.  In this case, the group         %
% 'fleets' is associated with variables simply because that group has     %
% fewer elements than the species group (number of fleets = 8, number of  %
% species = 10). Had the magnitudes been switched, the species would be   %
% associated with the variables and the fleets with sample entries.       %
%                                                                         %
% Also taken in as input, this code requires two vectors.  Working from   %
% our example, these vectors are measurements associated with our fleets, %
% and thus each have eight (8) entries, and in order to be used in this   %
% code's calculations, they must be column vectors.  The first vector     % 
% needs to contain what Collie et al. (2003) refers to as a 'status quo'  %
% measurement.  We will also need a 'scenario' measurement vector to have %
% something to compare to the 'status quo'.                               %
%                                                                         %
% The vector division of these two vectors (i.e., scenario vector divided %
% by status quo vector) produces the length of the vectors we plot in the %
% figure.  These eight (8) vectors, one for each fleet, show the relative %
% affect the scenario had on the system (which we assume would be at      %
% status quo in lieu of the perturbation implied by said scenario).  If   %
% there is no effect due to the scenario, the vector will be of length    %
% one (1) and will lie on the reference circle, which is of radius        %
% one (1).                                                                %
%                                                                         %
% The direction in which the plotted vector is pointing is caclulated     %
% from output produced by princomp.m and will point in the same direction %
% as the species plotted vectors associated with those fleets which arise %
% from running SampEntriesAMOEBA.m.                                       %
%                                                                         %
% For convenience, included in this package of .m files is the parent     %
% code BothAMOEBAS.m which run VariablesAMOEBA.m and SampEntriesAMOEBA.m  %
% simulaniously and produces AMOEBA plots for variables and sample        %
% entries (fleets and species, respectively) over a single scenario.  To  %
% run BothAMOEBAS.m, of course one will require status quo and scenario   %
% measurement vectors for both variables (fleets) and sample entries      %
% (species).                                                              %
%                                                                         %
% For more informantion, including an example with an example data matrix %
% and example data vectors, see the AMOEBAReadMe PDF document included    %
% with this code.                                                         %
%                                                                         %
% VariablesAMOEBA.m                                                       %
% Written by: Iris A. Gray, M.S.                                          %
%             Marine Ecosystem Modeller                                   %
%             NWFSC (NOAA)                                                %
%             Seattle, WA                                                 %
%             Iris.Gray@noaa.gov                                          %
%             9/9/2010                                                    %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% First things first, lets assign names to your variables that you want to 
% see in the figure that this code produces.  The three letters you see 
% after the 'f.' will be the name printed on its corresponding vector in 
% the figure.  To clarify further, the way this code is currently 
% structured, we have eight fleets named beamtrawl, fixedgear, industrial, 
% pelagic, saithe, seinenet, trawl, and otherfleets.  Change these names to
% whatever you desire.  Also, feel free to add or remove names as 
% necessary, but be sure to maintain the structural convention.

f.beamtrawl = 1;
f.fixedgear = 2;
f.industrial = 3;
f.pelagic = 4;
f.saithe = 5;
f.seinenet = 6;
f.trawl = 7;
f.otherfleets = 8;


% Here we make sure that the user entered their data matrix correctly,
% i.e., that the number of rows is greater than or equal to the number of
% columns.  If the user enters a matrix in which their are fewer rows than
% columns, an error message is displayed which includes a solution to the
% error.

[r,c] = size(M);

if r<c
    error('User has entered a matrix of size n-by-p such that n < p.  Our PCA calculator (princomp.m) requires that the input matrix must have at least as many rows as columns.  To fix the error, transpose the input matrix.  Be sure to match the names you assigned to your variables and sample entries accordingly.');
end


% From MATLAB's princomp.m (rewritten to fit the current circumstances): 
% The command princomp performs principal components analysis on the n-by-p 
% data matrix M, and calculates the principal component 
% coefficients, we call variables. Rows of M correspond to 
% observations, columns to variables. variables is a p-by-p matrix, each 
% column containing coefficients for one principal component. The columns 
% are in order of decreasing component variance. Also provided here are 
% sample entries, the principal component sample entries; that is, the representation of 
% M in the principal component space. Rows of sample entries correspond to 
% observations, columns to components.  Lastly, we have coveig, a vector 
% containing the eigenvalues of the covariance matrix of M.\
% Note: since sample entries are not pertinent to calculations or producing figures
% for this code, we represent it as we would any superfluous data and
% denote it with a 'junk'.


[variables, junk, coveig] = princomp(M);


% Here we caclulate the percentage of variance explained by our
% first two principal components.

decimalExp = cumsum(coveig)./sum(coveig);
decimalExp = decimalExp(2);


% Vector stuff.

p0 = [0,0];  % Initial point of every vector.
n = length(variables);  % Calculates number of rows for variables matrix.


% In order to draw a dashed outline around the vector endpoints, we must
% first order them so that patch.m knows what order to draw the vertices.
% This bit first converts all the variables into polar form and then grabs
% all the thetas and throws them into the first column of an n-by-2 matrix 
% called polarVector.

variables = variables(:,1:2);
for p = 1:n
    [theta,junk] = cart2pol(variables(p,1),variables(p,2));
    polarVector(p,1) = theta;
end


% Here we are caclulating the lengths of the vectors in which we will plot.

polarVector(:,2) = V_variablesScenario./V_variablesStatusQuo;

% This counts how many positive and negative thetas we have which aide in
% ordering vector endpoints properly.

pos = 0;
neg = 0;

for u = 1:n
    if polarVector(u,1)>0
        pos = pos + 1;
    else
        neg = neg + 1;
    end
end


% When we produce the plotted vectors, we want the ordering of them to 
% match the ordering of the names we gave them at the beginning of this 
% code.  Thus, newvariables is the matrix of x and y values which are the 
% plotted vector endpoints in the order corresponding to their vector 
% names.

for i  = 1:n
    [xCART1,yCART1] = pol2cart(polarVector(i,1), polarVector(i,2));
    newvariables(i,1) = xCART1;
    newvariables(i,2) = yCART1;
end


% Here, we seperate our positive and negative variables again, for the
% purpose of ordering them properly.  This needs to be done so 'patch' can
% draw the polygon around the vector endpoints properly.

for i = 1:n
    if polarVector(i,1)>0
        posvariables(i,1:2) = polarVector(i,1:2);
    else
        posvariables(i,1:2) = [0,0];
    end
    if variables(i,2)<0
        negvariables(i,1:2) = polarVector(i,1:2);
    else 
        negvariables(i,1:2) = [0,0];
    end

    
end 


% Shrinking down our vectors to their nonzero elements.

pPL = zeros(pos,2);
nPL = zeros(neg,2);
pPL(:,1) = nonzeros(posvariables(:,1));
pPL(:,2) = nonzeros(posvariables(:,2));
nPL(:,1) = nonzeros(negvariables(:,1));
nPL(:,2) = nonzeros(negvariables(:,2));


% Sorting our polarVector entries according to theta value for both 
% positive and negative theta values seperately.  Each theta's
% corresponding rho (entries resulting from the vector division we saw in 
% line 128 of this code) is sorted according to the sorted placement of its
% theta.

  [junk, NewRowNumberP] = sort(pPL(:,1));
  sortedPPL = pPL(NewRowNumberP,:);
  [junk, NewRowNumberN] = sort(nPL(:,1));
  sortedNPL = nPL(NewRowNumberN,:);

  
% Now, we can throw all of our sortedPPL and sortedNPL together into 
% sortedPL such that they are ordered according to their theta values, so 
% that they sweep around the origin from 0 to 2*pi.

  sortedPL = zeros(n,2);
  sortedPL(1:pos,:) = sortedPPL;
  sortedPL(pos+1:n,:) = sortedNPL;

% In order to plot these sorted polar coordinates in a Cartesian plane, we
% must convert:
  
for i  = 1:n
    [xCart1,yCart1] = pol2cart(sortedPL(i,1), sortedPL(i,2));
    [xCart2,yCart2] = pol2cart(sortedPL(i,1), 1);  % We additionally need 
                                                   % to caclulate what the
                                                   % AMOEBA area would be
                                                   % under the status quo
                                                   % scenaro.
    sortedPLCart(i,1) = xCart1;
    sortedPLCart(i,2) = yCart1;
    comparePLCart(i,1) = xCart2;                   % Status quo comparison 
    comparePLCart(i,2) = yCart2;                   % ordered vector
                                                   % endpoints.
    
end
  
% Here we calculate the area of the polygon (or AMOEBA) and also what 
% percentage of its status quo AMOEBA's area it encompasses.  You may want 
% to change 'fleet' inappropriate for your data set.
  
  variablesAmoebaArea = polyarea(sortedPLCart(:,1),sortedPLCart(:,2));
  statqVariablesAmoebaArea = polyarea(comparePLCart(:,1),comparePLCart(:,2));
  h = 0; k = 0; r = 1;    % Circle stuff.
  N = 256;                % Circle stuff.
  t = (0:N)*2*pi/N;       % Circle stuff.
  variablesComparitiveArea = variablesAmoebaArea/statqVariablesAmoebaArea;

  
% Now we are ready to make the variables figure!  Horray!

f1 = figure(1);


% This bit loads the names of the variables we chose at the beginning of the
% code.

fn = fieldnames(f);


% This command, 'patch' is what draws the polygon (or AMOEBA) whose 
% verticies are the endpoints of the plotted vectors.  As mentioned
% before, we use sortedPLCart as opposed to newvariables to draw the patch 
% since the vector endpoints are ordered in a way that they sweep around
% the origin.  Using newvariables, although having the same values as 
% sortedPLCart, would more often than not draw a crazy looking polygon
% since the vertices of the shape would probably not be drawn in the correct
% order.

patch(sortedPLCart(:,1),sortedPLCart(:,2), 'w', 'EdgeColor', 'k', 'LineStyle', '--')


% Title of the figure.  It's pretty generic so remember to change this to
% whatever seems appropriate for your data set.

title('Fleets AMOEBA', 'FontSize', 12)


% Here, we draw the reference circle and plot our variables vectors, with 
% names.  Please note that the placement of the names might not be 
% aesthetically pleasing in the figure produced.  If you wish to move the 
% name of the vector, then within the figure space, simply go to 'edit' and 
% then 'figure properties'which will allow you to move the vector names 
% wherever you please.



for i = 1:n

    hold on
    plot(r*cos(t) + h, r*sin(t) + k);  % Plots the reference circle
    vectarrow(p0, [newvariables(i,1), newvariables(i,2)]);
    
    % Plots vectors and corresponding names according to quadrant of the
    % Cartesian plane.
    
    %if newvariables(i,2)>0 & newvariables(i,1)>0
        text(newvariables(i,1), newvariables(i,2), fn(i))
    %elseif newvariables(i,2)<0 & newvariables(i,1)>0
    %    text(newvariables(i,1), newvariables(i,2)+0.05, fn(i))
    %elseif newvariables(i,2)<0 & newvariables(i,1)<0
    %    text(newvariables(i,1)-0.21, newvariables(i,2), fn(i))
    %elseif newvariables(i,2)>0 & newvariables(i,1)<0
    %    text(newvariables(i,1)-0.25, newvariables(i,2)+0.05, fn(i))
    %end
    grid off
    axis off
end


% This command saves the figure produced as variablesAMOEBA.pdf.  Please
% note that the figure may not pop up with a perfectly round reference
% circle, and therefore, the .pdf of the figure will then be an exact
% snapshot of whatever the figure looks like initially, which may be 
% undesired if the reference circle comes out looking like an elipse.  This
% can be easily remedied by stretching the sides of the figure viewing 
% window so that the reference circle looks circular, rather than 
% eliptical.  Then, save the figure, choosing the name that the figure was 
% already given (variablesAMOEBA.pdf).  You will then be prompted to 
% override the previous version, in which you definitely want to do so.

print(f1,'-dpdf','VariablesAMOEBA.pdf')